@@ -20,8 +20,7 @@
 # Portions created by the Initial Developer are Copyright (C) 2008, 2009
 # the Initial Developer. All Rights Reserved.
 #
-# Contributor(s): Changwoo Ryu <cwryu@debian.org>
-# Namhyung Kim <namhyung@gmail.com>
+# Contributor(s): See CREDITS file
 #
 # Alternatively, the contents of this file may be used under the terms of
 # either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -57,8 +56,8 @@ ALPHA_ALL = ''.join(map(unichr, range(ord('0'),ord('9')+1) + range(ord('a'),ord(
 COND_ALL = '.'
 COND_V_ALL = '[%s]' % (V_ALL + ALPHA_ALL)
 COND_T_ALL = '[%s]' % (T_ALL + ALPHA_ALL)
-COND_V_OR_RIEUL = '[%s]' % (V_ALL + u'\u11af' + ALPHA_ALL)
-COND_T_NOT_RIEUL = '[%s]' % (T_ALL.replace(u'\u11af', '') + ALPHA_ALL)
+COND_V_OR_RIEUL = '[%s]' % (V_ALL + T_RIEUL + ALPHA_ALL)
+COND_T_NOT_RIEUL = '[%s]' % (T_ALL.replace(T_RIEUL, '') + ALPHA_ALL)
 
 class JosaClass:
     next_flag = josas_flag_start
@@ -73,6 +72,13 @@ class JosaClass:
             if (word, '#' + pos) in self.notafter:
                 return False
         if self.after:
+            # 부사는 상태부사, 성상부사, 정도부사, 양태부사만 보조사 허용
+            if pos.startswith('부사'):
+                if pos in ['부사:상태', '부사:성상', '부사:정도', '부사:양태']:
+                    pos = '부사'
+                else:
+                    return False
+
             if ('#' + pos) in self.after:
                 return True
             elif (word, '#' + pos) in self.after:
@@ -104,12 +110,6 @@ groups['이'] = [
     JosaClass(
         rules = [(u'이', COND_T_ALL, '')],
         after = ['#명사', '#대명사'],
-        notafter = [('거', '#대명사'),
-                    ('그거', '#대명사'),
-                    ('요거', '#대명사'),
-                    ('이거', '#대명사'),
-                    ('저거', '#대명사'),
-                   ],
     ),
     # 대명사 '-거'+'이' -> '게'
     JosaClass(
@@ -126,29 +126,43 @@ groups['이'] = [
 # '가' 주격/보격 조사
 groups['가'] = [
     JosaClass(
-        rules = [(u'가', COND_ALL, '')],
+        rules = [(u'가', COND_V_ALL, '')],
         after = ['#명사', '#대명사',
                  '#특수:숫자', '#특수:알파벳',
                  '#특수:수:1', '#특수:수:10', '#특수:수:100', '#특수:수:1000'],
         notafter = [('나', '#대명사'),
-                    ('너', '#대명사')],
+                    ('너', '#대명사'),
+                    ('저', '#대명사'),
+                   ],
     ),
-    # 대명사 '나'+'가' -> '내가', '너'+가 -> '네가'
+    # 대명사 '나'+'가' -> '내가', '너'+'가' -> '네가', '저'+'가' -> '제가'
     JosaClass(
-        rules = [(V_AE + u'가', u'나', V_A),
-                 (V_E + u'가', u'너', V_EO)],
+        rules = [(V_AE + u'가', V_A, V_A),
+                 (V_E + u'가', V_EO, V_EO),
+                 ],
         after = [('나', '#대명사'),
-                 ('너', '#대명사')],
+                 ('너', '#대명사'),
+                 ('저', '#대명사'),
+                ],
+    ),
+    # 대명사 '누구'+'가' -> '누가'
+    JosaClass(
+        rules = [(u'가', u'누구', u'구')],
+        after = [('누구', '#대명사')],
     ),
 ]
 
 groups['!대명사+의'] = [
-    # 대명사 '나'+'의' -> '내', '너'+의 -> '네'
+    # 대명사 '나'+'의' -> '내', '너'+'의' -> '네', '저'+'의' -> '제'
     JosaClass(
-        rules = [(V_AE, u'나', V_A),
-                 (V_E, u'너', V_EO)],
+        rules = [(V_AE, V_A, V_A),
+                 (V_E, V_EO, V_EO),
+                 (V_E, V_EO, V_EO),
+                ],
         after = [('나', '#대명사'),
-                 ('너', '#대명사')],
+                 ('너', '#대명사'),
+                 ('저', '#대명사'),
+                ],
     ),
 ]
 
@@ -164,15 +178,12 @@ groups['!종성줄임'] = [
 # '거'+'로' => '걸로'
 groups['!거+로'] = [
     JosaClass(
-        rules = [(T_RIEUL + u'로', u'거', ''),
-                 (T_RIEUL + u'로는', u'거', ''),
-                 (T_RIEUL + u'론', u'거', ''),
-                 (T_RIEUL + u'로도', u'거', ''),
-                 (T_RIEUL + u'로서', u'거', ''),
-                 (T_RIEUL + u'로만', u'거', ''),
-                 (T_RIEUL + u'로써', u'거', ''),
-                 (T_RIEUL + u'로부터', u'거', ''),
-                 ],
+        rules = [(T_RIEUL + u'로' + emph, u'거', '')
+                 for emph in ['', u'는', T_RIEUL, u'도',
+                              u'만',
+                              u'서', u'서는', u'선', u'서도',
+                              u'써', u'써는', u'썬', u'써도',
+                              u'부터', u'부터는', u'부턴', u'부터도']],
         after = [('거', '#대명사'),
                  ('그거', '#대명사'),
                  ('요거', '#대명사'),
@@ -182,12 +193,28 @@ groups['!거+로'] = [
     ),
 ]
 
+# 보조사
+groups['!보조사'] = [
+    JosaClass(
+        rules = [('도', COND_ALL, ''),
+                 ('만', COND_ALL, ''),
+                 ('은', COND_T_ALL, ''), ('는', COND_V_ALL, '')],
+        after = ['#부사',
+                 '#명사', '#대명사', '#수사',
+                 '#특수:숫자', '#특수:알파벳',
+                 '#특수:수:1', '#특수:수:10', '#특수:수:100', '#특수:수:1000'],
+    ),
+]
+
 groups['*'] = [
     JosaClass(
         rules = [
          ('을', COND_T_ALL, ''), ('를', COND_V_ALL, ''),
          ('과', COND_T_ALL, ''), ('와', COND_V_ALL, ''),
-         ('은', COND_T_ALL, ''), ('는', COND_V_ALL, ''),
+         ('과는', COND_T_ALL, ''), ('와는', COND_V_ALL, ''),
+         ('관', COND_T_ALL, ''), ('완', COND_V_ALL, ''),
+         ('과도', COND_T_ALL, ''), ('와도', COND_V_ALL, ''),
+         ('과의', COND_T_ALL, ''), ('와의', COND_V_ALL, ''),
          ('라', COND_V_ALL, ''), ('이라', COND_T_ALL, ''),
          ('라고', COND_V_ALL, ''), ('이라고', COND_T_ALL, ''),
          ('라는', COND_V_ALL, ''), ('이라는', COND_T_ALL, ''),
@@ -201,9 +228,18 @@ groups['*'] = [
          ('론', COND_V_OR_RIEUL, ''), ('으론', COND_T_NOT_RIEUL, ''),
          ('로도', COND_V_OR_RIEUL, ''), ('으로도', COND_T_NOT_RIEUL, ''),
          ('로서', COND_V_OR_RIEUL, ''), ('으로서', COND_T_NOT_RIEUL, ''),
+         ('로서는', COND_V_OR_RIEUL, ''), ('으로서는', COND_T_NOT_RIEUL, ''),
+         ('로서도', COND_V_OR_RIEUL, ''), ('으로서도', COND_T_NOT_RIEUL, ''),
+         ('로선', COND_V_OR_RIEUL, ''), ('으로선', COND_T_NOT_RIEUL, ''),
          ('로만', COND_V_OR_RIEUL, ''), ('으로만', COND_T_NOT_RIEUL, ''),
          ('로써', COND_V_OR_RIEUL, ''), ('으로써', COND_T_NOT_RIEUL, ''),
+         ('로써는', COND_V_OR_RIEUL, ''), ('으로써는', COND_T_NOT_RIEUL, ''),
+         ('로썬', COND_V_OR_RIEUL, ''), ('으로썬', COND_T_NOT_RIEUL, ''),
+         ('로써도', COND_V_OR_RIEUL, ''), ('으로써도', COND_T_NOT_RIEUL, ''),
          ('로부터', COND_V_OR_RIEUL, ''), ('으로부터', COND_T_NOT_RIEUL, ''),
+         ('로부터는', COND_V_OR_RIEUL, ''), ('으로부터는', COND_T_NOT_RIEUL, ''),
+         ('로부턴', COND_V_OR_RIEUL, ''), ('으로부턴', COND_T_NOT_RIEUL, ''),
+         ('로부터도', COND_V_OR_RIEUL, ''), ('으로부터도', COND_T_NOT_RIEUL, ''),
          # sorted list
          ('같이', COND_ALL, ''),
          ('까지', COND_ALL, ''),
@@ -214,6 +250,7 @@ groups['*'] = [
          ('까지라도', COND_ALL, ''),
          ('께', COND_ALL, ''),
          ('께는', COND_ALL, ''),
+         ('껜', COND_ALL, ''),
          ('께도', COND_ALL, ''),
          ('께서', COND_ALL, ''),
          ('께서는', COND_ALL, ''),
@@ -225,11 +262,15 @@ groups['*'] = [
          ('대론', COND_ALL, ''),
          ('대로만', COND_ALL, ''),
          ('대로의', COND_ALL, ''),
-         ('도', COND_ALL, ''),
+         # TODO: -더러 조사는 사람이나 동물 등에만 붙음
+         ('더러', COND_ALL, ''),
+         ('더러는', COND_ALL, ''),
+         ('더러만', COND_ALL, ''),
+         ('더런', COND_ALL, ''),
          ('마다', COND_ALL, ''),         # 보조사, '모두'
          ('마저', COND_ALL, ''),
          ('마저도', COND_ALL, ''),
-         ('만', COND_ALL, ''),
+         ('만의', COND_ALL, ''),
          ('만이', COND_ALL, ''),
          ('밖에', COND_ALL, ''),
          ('밖에는', COND_ALL, ''),
@@ -239,6 +280,8 @@ groups['*'] = [
          ('보단', COND_ALL, ''),
          ('보다도', COND_ALL, ''),
          ('부터', COND_ALL, ''),
+         ('부터는', COND_ALL, ''),
+         ('부턴', COND_ALL, ''),
          ('부터라도', COND_ALL, ''),
          ('서', COND_ALL, ''),           # '~에서' 준말
          ('에', COND_ALL, ''),
@@ -253,6 +296,7 @@ groups['*'] = [
          ('에게서만', COND_ALL, ''),
          ('에는', COND_ALL, ''),         # 에+'는' 보조사
          ('엔', COND_ALL, ''),           # '에는' 준말
+         ('에나', COND_ALL, ''),         # 에+'나' 보조사
          ('에도', COND_ALL, ''),         # 에+'도' 보조사
          ('에만', COND_ALL, ''),         # 에+'만' 보조사
          ('에서', COND_ALL, ''),
@@ -273,6 +317,9 @@ groups['*'] = [
          # TODO: -한테 조사는 사람이나 동물 등에만 붙음
          ('하고', COND_ALL, ''),         # 구어체
          ('한테', COND_ALL, ''),
+         ('한테는', COND_ALL, ''),
+         ('한텐', COND_ALL, ''),
+         ('한테도', COND_ALL, ''),
          ('한테서', COND_ALL, ''),
          ],
         after = ['#명사', '#대명사', '#수사',
