@@ -625,15 +625,16 @@ groups['-다'] = [
 groups['-는'] = [
     { 'rules': [[u'-는', u'[^%s]' % T_RIEUL, ''],
                 [u'-는', T_RIEUL, T_RIEUL]],
-      'after': ['#동사', '^.*있다$', '^.*없다$', '^.*계시다$', '-으시-', '-겠-'],
+      'after': ['#동사', '^.*%s다$' % (V_I + T_SSANGSIOS), '^.*없다$', '^.*계시다$', '-으시-', '-겠-'],
     },
 ]
 
 #### 연결: -게
 groups['-게'] = [
     { 'rules': [[u'-게', '', ''],
-                [u'-케', COND_VOICED + u'하', u'하'], # 하게 -> 케 준말
-                [u'-게', COND_UNVOICED + u'하', u'하'], # 하게 -> 게 준말
+                # ~하다 준말
+                [u'-케', COND_VOICED + u'하', u'하'], # 하게 -> 케
+                [u'-게', COND_UNVOICED + u'하', u'하'], # 하게 -> 게
                 ],
       'after': ['#용언', '-으시-'],
     },
@@ -653,7 +654,7 @@ groups['-은'] = [
                 [u'-' + T_NIEUN, T_RIEUL, T_RIEUL],
                 [u'-은', COND_T_NOT_RIEUL, '']],
       'after': ['#용언', '#이다', '-으시-'],
-      'notafter': ['^.*있다$', '^.*없다$'],
+      'notafter': ['^.*%s다$' % (V_I + T_SSANGSIOS), '^.*없다$'],
       'notcond': ['#ㄷ불규칙', '#ㅂ불규칙', '#ㅅ불규칙', '#ㅎ불규칙'],
     },
     # ㄷ불규칙
@@ -670,7 +671,9 @@ groups['-은'] = [
 #### 종결: -지
 groups['-지'] = [
     { 'rules': [[u'-지', '', ''],
-                [u'-치', u'하', u'하'], # 하지 -> 치 준말
+                # ~하다 준말
+                [u'-치', COND_VOICED + u'하', u'하'], # 하지 -> 치
+                [u'-지', COND_UNVOICED +  u'하', u'하'], # 하지 -> 지
                ],
       'after': ['#용언', '#이다', '-으시-', '-었-', '-겠-'],
     },
@@ -684,8 +687,11 @@ groups['-지'][0]['rules'].append([u'-죠', '', ''])
 groups['-지마는'] = [
     { 'rules': [[u'-지마는', '', ''],
                 [u'-지만', '', ''],
-                [u'-치만', u'하', u'하'],   # 하지만 -> 치만 준말
-                [u'-치마는', u'하', u'하'], # 하지마는 -> 치마는 준말
+                # ~하다 준말
+                [u'-치만', COND_VOICED + u'하', u'하'],   # 하지만 -> 치만
+                [u'-지만', COND_UNVOICED + u'하', u'하'], # 하지만 -> 지만
+                [u'-치마는', COND_VOICED + u'하', u'하'], # 하지마는 -> 치마는
+                [u'-지마는', COND_UNVOICED + u'하', u'하'], # 하지마는 -> 치마는
                 ],
       'after': ['#용언', '#이다', '-었-', '-겠-'],
     },
@@ -830,7 +836,9 @@ for klass in groups['-으려면']:
 #### 연결: -도록
 groups['-도록'] = [
     { 'rules': [[u'-도록', '', ''],
-                [u'-토록', u'하', u'하'], # 하도록 -> 토록 준말
+                # ~하다 준말
+                [u'-토록', COND_VOICED + u'하', u'하'], # 하도록 -> 토록
+                [u'-도록', COND_UNVOICED + u'하', u'하'], # 하도록 -> 도록
                 ],
       'after': ['#동사', '-으시-',
                 '#형용사', # FIXME: 일부 형용사만 허용하지만 구분하기에는 너무 많다.
@@ -947,7 +955,9 @@ attach_emphasis(groups['-자고'], ['요'])
 #### 연결: -기로
 groups['-기로'] = [
     { 'rules': [[u'-기로', '', ''],
-                [u'-키로', u'하', u'하'], # 하기로 -> 키로 준말
+                # ~하다 준말
+                [u'-키로', COND_VOICED + u'하', u'하'], # 하기로 -> 키로
+                [u'-기로', COND_UNVOICED + u'하', u'하'], # 하기로 -> 기로
                 ],
       'after': ['#용언', '#이다', '-'],
       'notafter': ['-더-', '-으리-'],
@@ -1044,7 +1054,9 @@ attach_emphasis(groups['-은데'], ['도', '요'])
 #### 명사형 전성: -기
 groups['-기'] = [
     { 'rules': [[u'-기', '', ''],
-                [u'-키', u'하', u'하'], # 하기 -> 키 준말
+                # ~하다 준말
+                [u'-키', COND_VOICED + u'하', u'하'], # 하기 -> 키
+                [u'-기', COND_UNVOICED + u'하', u'하'], # 하기 -> 기
                ],
       'after': ['#용언', '#이다', '-으시-', '-었-', '-겠-'],
     },
@@ -1898,7 +1910,8 @@ groups['-을라'] = [
 #### 연결: -건대
 groups['-건대'] = [
     { 'rules': [[u'-건대', '', ''],
-                [u'-컨대', COND_VOICED + u'하', u'하'], # 하건대 -> 건대
+                # ~하다 준말
+                [u'-컨대', COND_VOICED + u'하', u'하'], # 하건대 -> 컨대
                 [u'-건대', COND_UNVOICED + u'하', u'하'], # 하건대 -> 건대
                 ],
       'after': ['#동사'],
